Package: fedstatAPIr
Title: Unofficial API for Fedstat (Rosstat EMISS System) for Automatic
        and Efficient Data Queries
Version: 1.0.3
Authors@R: c(
    person(given = "Denis",
           family = "Krylov",
	       role =c("aut", "cre"),
           email = "deniskrylovvit@gmail.com"),
    person(given = "Dmitry",
           family = "Kibalnikov",
	       role =c("aut"),
           email = "d.kibalnikov@gmail.com")     
    )  
Description: An API for automatic data queries to the fedstat <https://www.fedstat.ru>, using a small set of functions with a common interface.
License: MIT + file LICENSE
URL: https://github.com/DenchPokepon/fedstatAPIr
BugReports: https://github.com/DenchPokepon/fedstatAPIr/issues
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.2.3
Imports: httr, jsonlite, xml2, readsdmx, magrittr, utils, methods,
        data.table
Depends: R (>= 2.10)
NeedsCompilation: no
Packaged: 2023-03-30 07:02:25 UTC; denis
Author: Denis Krylov [aut, cre],
  Dmitry Kibalnikov [aut]
Maintainer: Denis Krylov <deniskrylovvit@gmail.com>
Repository: CRAN
Date/Publication: 2023-03-30 08:30:02 UTC
Built: R 4.6.0; ; 2026-04-28 03:16:04 UTC; unix
