Package: fedstatAPIr
Title: Unofficial API for Fedstat (Rosstat EMISS System) for Automatic
        and Efficient Data Queries
Version: 1.1.0
Authors@R: c(
    person(given = "Denis",
           family = "Krylov",
	       role =c("aut", "cre"),
           email = "deniskrylovvit@gmail.com"),
    person(given = "Dmitry",
           family = "Kibalnikov",
	       role =c("aut"),
           email = "d.kibalnikov@gmail.com")     
    )  
Description: An API for automatic data queries to the fedstat <https://www.fedstat.ru>, using a small set of functions with a common interface.
License: MIT + file LICENSE
URL: https://github.com/DenchPokepon/fedstatAPIr
BugReports: https://github.com/DenchPokepon/fedstatAPIr/issues
Encoding: UTF-8
LazyData: true
Imports: httr, jsonlite, xml2, readsdmx, magrittr, utils, methods,
        data.table
Depends: R (>= 3.5.0)
Config/roxygen2/version: 8.1.0
NeedsCompilation: no
Packaged: 2026-09-04 01:29:22 UTC; denis
Author: Denis Krylov [aut, cre],
  Dmitry Kibalnikov [aut]
Maintainer: Denis Krylov <deniskrylovvit@gmail.com>
Repository: CRAN
Date/Publication: 2026-09-04 02:30:02 UTC
