# Generated by roxygen2: do not edit by hand

export("%>%")
export(fedstat_data_ids_filter)
export(fedstat_data_load_with_filters)
export(fedstat_get_data_ids)
export(fedstat_indicator_info)
export(fedstat_parse_sdmx_to_table)
export(fedstat_post_data_ids_filtered)
import(data.table)
importFrom(data.table,":=")
importFrom(data.table,.BY)
importFrom(data.table,.EACHI)
importFrom(data.table,.GRP)
importFrom(data.table,.I)
importFrom(data.table,.N)
importFrom(data.table,.NGRP)
importFrom(data.table,.SD)
importFrom(data.table,data.table)
importFrom(magrittr,"%>%")
importFrom(utils,globalVariables)
